/*
 * Copyright 2018 NXP
 * Copyright 2021, 2022 Sony Corporation
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <spl.h>
#include <asm/io.h>
#include <errno.h>
#include <asm/io.h>
#include <asm/mach-imx/iomux-v3.h>
#include <asm/arch/imx8mm_pins.h>
#include <asm/arch/sys_proto.h>
#include <power/pmic.h>
#include <power/bd71837.h>
#include <asm/arch/clock.h>
#include <asm/mach-imx/gpio.h>
#include <asm/mach-imx/mxc_i2c.h>
#include <fsl_esdhc.h>
#include <mmc.h>
#include <asm/arch/imx8m_ddr.h>

DECLARE_GLOBAL_DATA_PTR;

#ifdef CONFIG_IMX8M_LPDDR4_MICRON
extern struct dram_timing_info dram_timing_micron;
#endif

void spl_dram_init(void)
{
#ifndef CONFIG_IMX8M_LPDDR4_MICRON
	ddr_init(&dram_timing);
#else
	ddr_init(&dram_timing_micron);
#endif
}

#define I2C_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PE)
#define PC MUX_PAD_CTRL(I2C_PAD_CTRL)
struct i2c_pads_info i2c_pad_info1 = {
	.scl = {
		.i2c_mode = IMX8MM_PAD_I2C1_SCL_I2C1_SCL | PC,
		.gpio_mode = IMX8MM_PAD_I2C1_SCL_GPIO5_IO14 | PC,
		.gp = IMX_GPIO_NR(5, 14),
	},
	.sda = {
		.i2c_mode = IMX8MM_PAD_I2C1_SDA_I2C1_SDA | PC,
		.gpio_mode = IMX8MM_PAD_I2C1_SDA_GPIO5_IO15 | PC,
		.gp = IMX_GPIO_NR(5, 15),
	},
};

#define USDHC2_CD_GPIO	IMX_GPIO_NR(2, 18)
#define USDHC2_PWR_GPIO IMX_GPIO_NR(2, 19)

#define USDHC_PAD_CTRL	(PAD_CTL_DSE6 | PAD_CTL_HYS | PAD_CTL_PUE |PAD_CTL_PE | \
			 PAD_CTL_FSEL2)
#define USDHC_GPIO_PAD_CTRL (PAD_CTL_HYS | PAD_CTL_DSE1)

static iomux_v3_cfg_t const usdhc3_pads[] = {
	IMX8MM_PAD_NAND_WE_B_USDHC3_CLK | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_WP_B_USDHC3_CMD | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA04_USDHC3_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA05_USDHC3_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA06_USDHC3_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_DATA07_USDHC3_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_RE_B_USDHC3_DATA4 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_CE2_B_USDHC3_DATA5 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_CE3_B_USDHC3_DATA6 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_NAND_CLE_USDHC3_DATA7 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
};

static iomux_v3_cfg_t const usdhc2_pads[] = {
	IMX8MM_PAD_SD2_CLK_USDHC2_CLK | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_CMD_USDHC2_CMD | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA0_USDHC2_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA1_USDHC2_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA2_USDHC2_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_DATA3_USDHC2_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	IMX8MM_PAD_SD2_RESET_B_GPIO2_IO19 | MUX_PAD_CTRL(USDHC_GPIO_PAD_CTRL),
};

/*
 * The evk board uses DAT3 to detect CD card plugin,
 * in u-boot we mux the pin to GPIO when doing board_mmc_getcd.
 */
static iomux_v3_cfg_t const usdhc2_cd_pad =
	IMX8MM_PAD_SD2_DATA3_GPIO2_IO18 | MUX_PAD_CTRL(USDHC_GPIO_PAD_CTRL);

static iomux_v3_cfg_t const usdhc2_dat3_pad =
	IMX8MM_PAD_SD2_DATA3_USDHC2_DATA3 |
	MUX_PAD_CTRL(USDHC_PAD_CTRL);


static struct fsl_esdhc_cfg usdhc_cfg[2] = {
	{USDHC2_BASE_ADDR, 0, 1},
	{USDHC3_BASE_ADDR, 0, 1},
};

int board_mmc_init(bd_t *bis)
{
	int i, ret;
	/*
	 * According to the board_mmc_init() the following map is done:
	 * (U-Boot device node)    (Physical Port)
	 * mmc0                    USDHC1
	 * mmc1                    USDHC2
	 */
	for (i = 0; i < CONFIG_SYS_FSL_USDHC_NUM; i++) {
		switch (i) {
		case 0:
			usdhc_cfg[0].sdhc_clk = mxc_get_clock(MXC_ESDHC2_CLK);
			imx_iomux_v3_setup_multiple_pads(
				usdhc2_pads, ARRAY_SIZE(usdhc2_pads));
			gpio_request(USDHC2_PWR_GPIO, "usdhc2_reset");
			gpio_direction_output(USDHC2_PWR_GPIO, 0);
			udelay(500);
			gpio_direction_output(USDHC2_PWR_GPIO, 1);
			break;
		case 1:
			usdhc_cfg[1].sdhc_clk = mxc_get_clock(MXC_ESDHC3_CLK);
			imx_iomux_v3_setup_multiple_pads(
				usdhc3_pads, ARRAY_SIZE(usdhc3_pads));
			break;
		default:
			printf("Warning: you configured more USDHC controllers"
				"(%d) than supported by the board\n", i + 1);
			return -EINVAL;
		}

		ret = fsl_esdhc_initialize(bis, &usdhc_cfg[i]);
		if (ret)
			return ret;
	}

	return 0;
}

int board_mmc_getcd(struct mmc *mmc)
{
	struct fsl_esdhc_cfg *cfg = (struct fsl_esdhc_cfg *)mmc->priv;
	int ret = 0;

	switch (cfg->esdhc_base) {
	case USDHC3_BASE_ADDR:
		ret = 1;
		break;
	case USDHC2_BASE_ADDR:
		imx_iomux_v3_setup_pad(usdhc2_cd_pad);
		gpio_request(USDHC2_CD_GPIO, "usdhc2 cd");
		gpio_direction_input(USDHC2_CD_GPIO);

		/*
		 * Since it is the DAT3 pin, this pin is pulled to
		 * low voltage if no card
		 */
		ret = gpio_get_value(USDHC2_CD_GPIO);

		imx_iomux_v3_setup_pad(usdhc2_dat3_pad);
		return ret;
	}

	return 1;
}

#ifdef CONFIG_POWER
#define I2C_PMIC	0
int power_init_board(void)
{
	struct pmic *p;
	int ret;
	u32 val;

	ret = power_bd71837_init(I2C_PMIC);
	if (ret)
		printf("power init failed");

	p = pmic_get("BD71837");
	pmic_probe(p);

	ret = pmic_reg_read(p, 0x09, &val);
	if (ret != 0)
		printf("--> Fail to read buck5 register(0x09)\n");

	printf("buck5 0x%x\n", val);

	if (val == 0x02 /* PDVPUGPUDRAM */) {
		ret = pmic_reg_write(p, 0x09, 0x00);
		if (ret != 0)
			printf("--> Fail to write buck5 register\n");
		udelay(1000); /* wait a stable of switching regulator */
	}

#ifdef CONFIG_ICX_ERRATA_BD71837_REVA0
	/* BD71837 bug workaround for rev.A0 */
  {
	unsigned char addr_org = pmic_i2c_addr;
	u32 regval;
	int rval;

	printf("BD71837: i2c_addr = 0x%x\n", addr_org);

	if (pmic_reg_read(p, BD71837_REV, &regval) != 0)
		printf("power_init_board: cannot read PMIC REV register.\n");

	printf("BD71837: revision(0x%x)\n", regval);
	do {
		if (regval == 0xa0) {
			printf("BD71837 need a workaround\n");

			/* Try to enable slave addr 0x5c/5d/5e */
			rval = pmic_reg_write(p, 0xfe, 0x8c);
			if (rval != 0) {
				printf("--> Fail to enter debug mode(1)\n");
				break;
			}

			/*
			 * Try to enter test mode with alternate slave addr
			 * BUS(0):ADDR(0x5e):REG(0) enter into test mode
			 */
			pmic_i2c_addr = 0x5e;
			rval = pmic_reg_write(p, 0x0, 0x1);
			if (rval != 0) {
				printf("--> Fail to enter debug mode(2)\n");
				break;
			}

			/*
			 * write REG[0x5E:7F] to 0x57;
			 */
			rval = pmic_reg_read(p, 0x7f, &regval);
			if (rval != 0) {
				printf("--> Fail to read debug register(0x5E:0x7F)\n");
				break;
			}
			printf("REG[0x5E:0x7F]: 0x%x ->", regval);
			rval = pmic_reg_write(p, 0x7f, 0x57);
			if (rval != 0) {
				printf("\n--> Fail to write workaround (0x5E:0x7F)=0x57\n");
				break;
			}
			rval = pmic_reg_read(p, 0x7f, &regval);
			if (rval != 0) {
				printf("--> Fail to read debug register(0x5E:0x7F) again\n");
				break;
			}
			printf("0x%x\n", regval);

			if (regval != 0x57) {
				printf("--> Fail to verify workaround.\n");
				break;
			}
			printf("BD71837 applied a workaround\n");

			/* recover normal i2c slave address */
			pmic_i2c_addr = addr_org;
			(void) pmic_reg_write(p, 0xfe, 0);
			(void) pmic_reg_read(p, BD71837_REV, &regval);
			if (regval != 0xa0) {
				printf("--> Fail to recover normal i2c slave address.\n");
			}
		} else {
			printf("BD71837 don't need a workaround\n");
		}
	} while (0);

	/* make sure normal slave addr in case of break */
	pmic_i2c_addr = addr_org;
  }
#endif /* ICX_ERRATA_BD71837_REVA0 */

	/* Increase RESET key long push time from the default 10ms to 3s */
	pmic_reg_write(p, BD71837_PWRONCONFIG1, 0x03);

	/* unlock the PMIC regs */
	pmic_reg_write(p, BD71837_REGLOCK, 0x1);

	/* increase VDD_SOC to typical value 0.82v before first DRAM access */
	pmic_reg_write(p, BD71837_BUCK1_VOLT_RUN, 0x0c);
	pmic_reg_write(p, BD71837_BUCK1_VOLT_IDLE, 0x0c);
	pmic_reg_write(p, BD71837_BUCK1_VOLT_SUSP, 0x0c);

	/* increase VDD_DRAM to 0.975v for 3Ghz DDR */
	pmic_reg_write(p, BD71837_BUCK5_VOLT, 0x83);

#ifndef CONFIG_IMX8M_LPDDR4
	/* increase NVCC_DRAM_1V2 to 1.2v for DDR4 */
	pmic_reg_write(p, BD71837_BUCK8_VOLT, 0x28);
#endif

	/* lock the PMIC regs */
	pmic_reg_write(p, BD71837_REGLOCK, 0x11);

	return 0;
}
#endif

#ifdef CONFIG_SPL_POWER_CONTROL
#define MODEL_ID1_GPIO	IMX_GPIO_NR(2,  5)
#define MODEL_ID2_GPIO	IMX_GPIO_NR(2,  4)
#define MODEL_ID3_GPIO	IMX_GPIO_NR(1, 15)
#define IOEXP_XRST_GPIO	IMX_GPIO_NR(1,  5)
#define IOEXP_INT_GPIO	IMX_GPIO_NR(1, 13)
#define PCONT_AMP_GPIO	IMX_GPIO_NR(2, 20)
#define PCONT_RY_GPIO	IMX_GPIO_NR(3, 21)
#define PCONT_ON_GPIO	IMX_GPIO_NR(3,  0)
#define BD_REQ_GPIO	IMX_GPIO_NR(3,  7)
#define IFCON_RST_GPIO	IMX_GPIO_NR(3,  9)
#define WATCHDOG_GPIO	IMX_GPIO_NR(3, 14)
#define ECSPI1_SCLK	IMX_GPIO_NR(5,  6)
#define ECSPI1_MISO	IMX_GPIO_NR(5,  7)
#define ECSPI1_MOSI	IMX_GPIO_NR(5,  8)
#define ECSPI1_SS0	IMX_GPIO_NR(5,  9)

#define PAD_CTRL_GPIO_DS0 MUX_PAD_CTRL(PAD_CTL_FSEL2 | PAD_CTL_DSE0)
#define PAD_CTRL_GPIO_DS1 MUX_PAD_CTRL(PAD_CTL_FSEL2 | PAD_CTL_DSE1)
#define PAD_CTRL_GPIO_DS7 MUX_PAD_CTRL(PAD_CTL_FSEL2 | PAD_CTL_DSE7)
#define PAD_CTRL_GPIO_PU_DS0 MUX_PAD_CTRL(PAD_CTL_PE | PAD_CTL_PUE | PAD_CTL_FSEL2 | PAD_CTL_DSE0)
#define PAD_CTRL_SPI_OUT MUX_PAD_CTRL(PAD_CTL_FSEL2 | PAD_CTL_DSE1)
#define PAD_CTRL_SPI_INP MUX_PAD_CTRL(PAD_CTL_PE | PAD_CTL_FSEL2 | PAD_CTL_DSE1)
#define PAD_CTRL_I2C MUX_MODE_SION | MUX_PAD_CTRL(PAD_CTL_FSEL2 | PAD_CTL_DSE1)
/* Pin-ctrl for ha-bb */
static iomux_v3_cfg_t const bb_ifcon_pads[] = {
	IMX8MM_PAD_GPIO1_IO05_GPIO1_IO5  | PAD_CTRL_GPIO_PU_DS0,	/* A53_Exp_RST */
	IMX8MM_PAD_GPIO1_IO13_GPIO1_IO13 | PAD_CTRL_GPIO_DS0,		/* A53_Exp_INT */
	IMX8MM_PAD_SD2_WP_GPIO2_IO20     | PAD_CTRL_GPIO_DS7,		/* PCONT_AMP */
	IMX8MM_PAD_NAND_ALE_GPIO3_IO0    | PAD_CTRL_GPIO_DS7,		/* PCONT_ON */
	IMX8MM_PAD_NAND_DATA01_GPIO3_IO7 | PAD_CTRL_GPIO_DS7,		/* BD_REQ */
	IMX8MM_PAD_NAND_DATA03_GPIO3_IO9 | PAD_CTRL_GPIO_DS7,		/* IF_RESET */
	IMX8MM_PAD_NAND_DQS_GPIO3_IO14   | PAD_CTRL_GPIO_DS7,		/* WATCHDOG */
	IMX8MM_PAD_ECSPI1_SCLK_GPIO5_IO6 | PAD_CTRL_SPI_OUT,		/* ECSPI1 SCLK */
	IMX8MM_PAD_ECSPI1_MOSI_GPIO5_IO7 | PAD_CTRL_SPI_OUT,		/* ECSPI1 MOSI */
	IMX8MM_PAD_ECSPI1_MISO_GPIO5_IO8 | PAD_CTRL_SPI_INP,		/* ECSPI1 MISO */
	IMX8MM_PAD_ECSPI1_SS0_GPIO5_IO9  | PAD_CTRL_SPI_OUT,		/* ECSPI1 SS0 */
};

/* Pin-ctrl for str model */
static iomux_v3_cfg_t const boardid_pads[] = {
	IMX8MM_PAD_SD1_DATA3_GPIO2_IO5   | PAD_CTRL_GPIO_DS0,		/* MODEL_ID1 */
	IMX8MM_PAD_SD1_DATA2_GPIO2_IO4   | PAD_CTRL_GPIO_DS0,		/* MODEL_ID2 */
	IMX8MM_PAD_GPIO1_IO15_GPIO1_IO15 | PAD_CTRL_GPIO_DS0,		/* MODEL_ID3 */
};
static iomux_v3_cfg_t const str_ifcon_pads[] = {
	IMX8MM_PAD_GPIO1_IO05_GPIO1_IO5  | PAD_CTRL_GPIO_PU_DS0,	/* A53_Exp_RST */
	IMX8MM_PAD_GPIO1_IO13_GPIO1_IO13 | PAD_CTRL_GPIO_DS0,		/* A53_Exp_INT */
	IMX8MM_PAD_SAI5_RXD0_GPIO3_IO21  | PAD_CTRL_GPIO_DS7,		/* PCONT_RY */
	IMX8MM_PAD_NAND_ALE_GPIO3_IO0    | PAD_CTRL_GPIO_DS7,		/* PCONT_ON */
	IMX8MM_PAD_NAND_DATA01_GPIO3_IO7 | PAD_CTRL_GPIO_DS7,		/* BD_REQ */
	IMX8MM_PAD_NAND_DATA03_GPIO3_IO9 | PAD_CTRL_GPIO_DS7,		/* IF_RESET */
	IMX8MM_PAD_NAND_DQS_GPIO3_IO14   | PAD_CTRL_GPIO_DS7,		/* WATCHDOG */
	IMX8MM_PAD_ECSPI1_SCLK_GPIO5_IO6 | PAD_CTRL_SPI_OUT,		/* ECSPI1 SCLK */
	IMX8MM_PAD_ECSPI1_MOSI_GPIO5_IO7 | PAD_CTRL_SPI_OUT,		/* ECSPI1 MOSI */
	IMX8MM_PAD_ECSPI1_MISO_GPIO5_IO8 | PAD_CTRL_SPI_INP,		/* ECSPI1 MISO */
	IMX8MM_PAD_ECSPI1_SS0_GPIO5_IO9  | PAD_CTRL_SPI_OUT,		/* ECSPI1 SS0 */
};

struct i2c_pads_info i2c_pad_info4 = {
	.scl = {
		.i2c_mode = IMX8MM_PAD_I2C4_SCL_I2C4_SCL    | PAD_CTRL_I2C,
		.gpio_mode = IMX8MM_PAD_I2C4_SCL_GPIO5_IO20 | PAD_CTRL_GPIO_DS1,
		.gp = IMX_GPIO_NR(5, 20),
	},
	.sda = {
		.i2c_mode = IMX8MM_PAD_I2C4_SDA_I2C4_SDA    | PAD_CTRL_I2C,
		.gpio_mode = IMX8MM_PAD_I2C4_SDA_GPIO5_IO21 | PAD_CTRL_GPIO_DS1,
		.gp = IMX_GPIO_NR(5, 21),
	},
};

#define TCA64XX_BUS (3)		/* i2c4 */
#define TCA6424_ADR (0x22)
#define TCA6424_OUT_REG (0x84)
#define TCA6424_DIR_REG (0x8C)
/* Initialize Data (HA-BB) */
static unsigned char bb_dir_init[] = { 0x00, 0xef, 0xff };
 /* All Off */
static unsigned char bb_out_init[] = { 0x00, 0x00, 0x00 };
 /* Set ENET_RST */
static unsigned char bb_out_pwr1[] = { 0x00, 0x10, 0x00 };
 /* Set PCONT1+PCONT_AMP+PCONT_TS+PCONT_MIC+ENET_RST */
static unsigned char bb_out_pwr2[] = { 0x27, 0x10, 0x00 };

/* Initialize Data (STR) */
static unsigned char str_dir_init[] = { 0x00, 0x08, 0x92 };
/* All Off */
static unsigned char str_out_init[] = { 0x00, 0x00, 0x00 };
/* Set PCONT1+PCONT_ETH */
static unsigned char str_out_pwr1[] = { 0x01, 0x04, 0x00 };
/* Set PCONT1+PCONT_ETH+ENET_RST */
static unsigned char str_out_pwr2[] = { 0x01, 0x14, 0x00 };
#define DELAY_UP_PWR1	(70000)
#define DELAY_UP_PWR2	(30000)
#define DELAY_UP_PWR3	(10000)

void ioexp_write(uint8_t adr, unsigned int reg, unsigned char *vals, int len)
{
	if (i2c_write(adr, reg, 1, vals, len))
		puts("i2c_write error\n");
}

int pcont_setup(void)
{
	int bid;

	imx_iomux_v3_setup_multiple_pads(boardid_pads,
					 ARRAY_SIZE(boardid_pads));
	gpio_request(MODEL_ID1_GPIO, "MODELID1");
	gpio_request(MODEL_ID2_GPIO, "MODELID2");
	gpio_request(MODEL_ID3_GPIO, "MODELID3");
	gpio_direction_input(MODEL_ID1_GPIO);
	gpio_direction_input(MODEL_ID2_GPIO);
	gpio_direction_input(MODEL_ID3_GPIO);
	bid = gpio_get_value(MODEL_ID1_GPIO)
	    | (gpio_get_value(MODEL_ID2_GPIO) << 1)
	    | (gpio_get_value(MODEL_ID3_GPIO) << 2);

	if (bid != 0)
		imx_iomux_v3_setup_multiple_pads(str_ifcon_pads,
						 ARRAY_SIZE(str_ifcon_pads));
	else
		imx_iomux_v3_setup_multiple_pads(bb_ifcon_pads,
						 ARRAY_SIZE(bb_ifcon_pads));
	if (setup_i2c(TCA64XX_BUS, CONFIG_SYS_I2C_SPEED, 0x7f, &i2c_pad_info4))
		puts("setup_i2c error\n");

	gpio_request(IOEXP_XRST_GPIO, "A53_Exp_RST");
	gpio_request(IOEXP_INT_GPIO, "A53_Exp_INT");
	gpio_request(IFCON_RST_GPIO, "IF_RESET");
	gpio_request(PCONT_ON_GPIO, "PCONT_ON");
	gpio_request(BD_REQ_GPIO, "BD_REQ");
	gpio_request(WATCHDOG_GPIO, "WATCHDOG");
	gpio_request(ECSPI1_SCLK, "ECSPI1_SCLK");
	gpio_request(ECSPI1_MOSI, "ECSPI1_MOSI");
	gpio_request(ECSPI1_MISO, "ECSPI1_MISO");
	gpio_request(ECSPI1_SS0, "ECSPI1_SS0");
	gpio_direction_output(IOEXP_XRST_GPIO, 0);
	gpio_direction_input(IOEXP_INT_GPIO);
	gpio_direction_output(IFCON_RST_GPIO, 1);
	gpio_direction_output(PCONT_ON_GPIO, 0);
	gpio_direction_output(BD_REQ_GPIO, 0);
	gpio_direction_output(WATCHDOG_GPIO, 0);
	gpio_direction_output(ECSPI1_SCLK, 0);
	gpio_direction_output(ECSPI1_MOSI, 0);
	gpio_direction_input(ECSPI1_MISO);
	gpio_direction_output(ECSPI1_SS0, 0);
	if (bid != 0) {
		gpio_request(PCONT_RY_GPIO, "PCONT_RY");
		gpio_direction_output(PCONT_RY_GPIO, 0);
	} else {
		gpio_request(PCONT_AMP_GPIO, "PCONT_AMP");
		gpio_direction_output(PCONT_AMP_GPIO, 0);
	}
	udelay(10000);
	gpio_set_value(IOEXP_XRST_GPIO, 1);
	gpio_set_value(IFCON_RST_GPIO, 0);
	if (bid != 0) {
		gpio_set_value(PCONT_RY_GPIO, 1);
		udelay(DELAY_UP_PWR1);
	}
	gpio_set_value(PCONT_ON_GPIO, 1);

	if (i2c_set_bus_num(TCA64XX_BUS))
		puts("i2c_set_bus_num error\n");
 
	if (i2c_probe(TCA6424_ADR))
		puts("i2c_probe error\n");

	if (bid != 0) {
		puts("PCONT startup str-set\n");
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, str_out_init, 3);
		ioexp_write(TCA6424_ADR, TCA6424_DIR_REG, str_dir_init, 3);
		udelay(DELAY_UP_PWR2);
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, str_out_pwr1, 3);
		udelay(DELAY_UP_PWR3);
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, str_out_pwr2, 3);
	} else {
		puts("PCONT startup ha-bb\n");
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, bb_out_init, 3);
		ioexp_write(TCA6424_ADR, TCA6424_DIR_REG, bb_dir_init, 3);
		udelay(10000);
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, bb_out_pwr1, 3);
		udelay(90000);
		ioexp_write(TCA6424_ADR, TCA6424_OUT_REG, bb_out_pwr2, 3);
		gpio_set_value(PCONT_AMP_GPIO, 1);
	}
	gpio_set_value(BD_REQ_GPIO, 1);

	return 0;
}
#endif

void spl_board_init(void)
{
#ifndef CONFIG_SPL_USB_SDP_SUPPORT
	/* Serial download mode */
	if (is_usb_boot()) {
		puts("Back to ROM, SDP\n");
		restore_boot_params();
	}
#endif
	puts("Normal Boot\n");
}

#ifdef CONFIG_SPL_LOAD_FIT
int board_fit_config_name_match(const char *name)
{
	/* Just empty function now - can't decide what to choose */
	debug("%s: %s\n", __func__, name);

	return 0;
}
#endif

void board_init_f(ulong dummy)
{
	int ret;

	/* Clear global data */
	memset((void *)gd, 0, sizeof(gd_t));

	arch_cpu_init();

	board_early_init_f();

	timer_init();

	preloader_console_init();

	/* Clear the BSS. */
	memset(__bss_start, 0, __bss_end - __bss_start);

	ret = spl_init();
	if (ret) {
		debug("spl_init() failed: %d\n", ret);
		hang();
	}

	enable_tzc380();

	/* Adjust pmic voltage to 1.0V for 800M */
	setup_i2c(0, CONFIG_SYS_I2C_SPEED, 0x7f, &i2c_pad_info1);

	power_init_board();
#ifdef CONFIG_SPL_POWER_CONTROL
	pcont_setup();
#endif
	/* DDR initialization */
	spl_dram_init();

	board_init_r(NULL, 0);
}
